from django.shortcuts import get_object_or_404
from api.models import User
from devices.models import Device
from api.notifications import send_sms
import os
import logging
from django.utils import timezone
from api.notifications import send_clean_telegram_markdown
from api.omada import Omada
from apibase.env import env, BASE_DIR
from procrastinate.contrib.django import app


logger = logging.getLogger(__name__)

env.read_env(os.path.join(BASE_DIR, ".env"))

omada_client = Omada()


@app.task
def add(x, y):
    print(f"Adding {x} and {y}")
    return x + y


@app.periodic(cron="0 0 */28 * *")  # type: ignore
@app.task
def deactivate_expired_devices():
    expired_devices = Device.objects.filter(
        expiry_date__lte=timezone.localtime(timezone.now()), is_active=True
    ).select_related("user")
    print("Expired Devices: ", expired_devices)
    count = expired_devices.count()
    user_devices_map = {}
    for device in expired_devices:
        if device.user and device.user.mobile:
            if device.user.mobile not in user_devices_map:
                user_devices_map[device.user.mobile] = []
            user_devices_map[device.user.mobile].append(device.name)

    for mobile, device_names in user_devices_map.items():
        if not mobile:
            continue
        device_list = "\n".join(
            [f"{i + 1}. {name}" for i, name in enumerate(device_names)]
        )
        print("device list: ", device_list)
        send_sms(
            mobile,
            f"Dear {mobile}, \n\nThe following devices have expired: \n{device_list}. \n\nPlease make a payment to keep your devices active. \n\n- SAR Link",
        )
    # expired_devices.update(is_active=False)
    print(f"Total {count} expired devices.")
    return {
        "total_expired_devices": count,
    }


@app.task
def add_new_devices_to_omada(new_devices: list[dict]):
    """
    Add new devices to Omada via Omada class.
    :param new_devices: List of new device names to add.
    """
    logger.info("Running add new devices to Omada task...")
    omada_client.add_new_devices_to_omada(new_devices)


def verify_user_with_person_api_task(user_id: int):
    """
    Verify the user with the Person API.
    :param user_id: The ID of the user to verify.
    """
    if not user_id:
        logger.error("User ID is not provided.")
        return None
    user = get_object_or_404(User, id=user_id)
    if not user:
        logger.error(f"User with ID {user_id} not found.")
        return None
    # Call the Person API to verify the user

    verification_failed_message = f"""
    _The following user verification failed_:
    *ID Card:* {user.id_card}
    *Name:* {user.first_name} {user.last_name}
    *House Name:* {user.address}
    *Date of Birth:* {user.dob}
    *Island:* {(user.atoll.name if user.atoll else "N/A")} {(user.island.name if user.island else "N/A")}
    *Mobile:* {user.mobile}
    Visit [SAR Link Portal](https://portal.sarlink.net) to manually verify this user.
    """

    # logger.info(verification_failed_message)
    PERSON_VERIFY_BASE_URL = env.str("PERSON_VERIFY_BASE_URL", default="")  # type: ignore

    if not PERSON_VERIFY_BASE_URL:
        raise ValueError(
            "PERSON_VERIFY_BASE_URL is not set in the environment variables."
        )
    import requests

    response = requests.get(f"{PERSON_VERIFY_BASE_URL}/api/person/{user.id_card}")
    if response.status_code == 200:
        data = response.json()
        api_nic = data.get("nic")
        api_name = data.get("name_en")
        api_house_name = data.get("house_name_en")
        api_dob = data.get("dob")
        api_atoll = data.get("atoll_en")
        api_island_name = data.get("island_name_en")

        if not user.mobile or user.dob is None:
            logger.error("User mobile or date of birth is not set.")
            return None
        if not user.island or user.atoll is None:
            logger.error("User island or atoll is not set.")
            return None

        logger.info(f"API nic: {api_nic}")
        logger.info(f"API name: {api_name}")
        logger.info(f"API house name: {api_house_name}")
        logger.info(f"API dob: {api_dob}")
        logger.info(f"API atoll: {api_atoll}")
        logger.info(f"API island name: {api_island_name}")

        user_nic = user.id_card
        user_name = f"{user.first_name} {user.last_name}"
        user_house_name = user.address
        user_dob = user.dob.isoformat()

        logger.info(f"User nic: {user_nic}")
        logger.info(f"User name: {user_name}")
        logger.info(f"User house name: {user_house_name}")
        logger.info(f"User dob: {user_dob}")
        logger.info(f"User atoll: {user.atoll}")
        logger.info(f"User island name: {user.island}")

        logger.info(
            f"case User atoll: {user.atoll.name == api_atoll.strip() if api_atoll else False}"
        )  # Defensive check for api_atoll
        logger.info(f"api atoll type: {type(api_atoll)}")
        logger.info(f"user atoll type: {type(user.atoll.name)}")
        logger.info(
            f"case User island name: {user.island.name == api_island_name.strip() if api_island_name else False}"
        )  # Defensive check for api_island_name
        logger.info(f"api island name type: {type(api_island_name)}")
        logger.info(f"user island name type: {type(user.island.name)}")

        if (
            data.get("nic") == user.id_card
            and data.get("name_en") == f"{user.first_name} {user.last_name}"
            and data.get("house_name_en") == user.address
            and data.get("dob").split("T")[0] == user.dob.isoformat()
            and data.get("atoll_en").strip() == user.atoll.name
            and data.get("island_name_en").strip() == user.island.name
        ):
            user.verified = True
            user.save()
            send_sms(
                user.mobile,
                f"Dear {user.first_name} {user.last_name}, \n\nYour account has been successfully and verified. \n\nYou can now manage your devices and make payments through our portal at https://portal.sarlink.net. \n\n - SAR Link",
            )
            return True
        else:
            user.verified = False
            user.save()

            send_sms(
                user.mobile,
                f"Dear {user.first_name} {user.last_name}, \n\nYour account registration is being processed. \n\nWe will notify you once verification is complete. \n\n - SAR Link",
            )
            send_clean_telegram_markdown(message=verification_failed_message)
            return False
    else:
        # Handle the error case
        print(
            f"Error verifying user with api: {response.status_code} - {response.text}"
        )
        return
